'
' Game of Life for use on the Sure 2416 LED dot matrix display
'
' This version uses a button to end a non-static display,
' and to start the initial generation, for a more random result.
'
' Written by Les Johnson for the Proton BASIC compiler on a PIC18F device
'
    Include "Amicus18.inc"                              ' Configure the compiler for an 18F25K20 at 64MHz. i.e. An Amicus18 board
    Include "Amicus18_Timers.inc"                       ' Load the Amicus18 timer macros into the program
    Include "2416.inc"                                  ' Load the Sure 2416 display interface routines into the program
'
' Declare variables used for the game of life
'
    Dim bDifferenceCount As Byte                        ' Indicates whether the display is static or not (0 if static)
    Dim bNeighbours As Byte                             ' Neighbour counter
    Dim Xpos As Byte                                    ' X position of display and shadow display RAM
    Dim Ypos As Byte                                    ' Y position of display And shadow display RAM
    Dim wRandResult As Word                             ' Result of the pseudo random number generator
    Dim bNeighbourState As Byte                         ' The display pixel's condition (0 or 1)
    Dim tShadowPixelColor As Byte                       ' The colour of the shadow display pixel (0 or 1)
'
' Declare variables used for the scrolling text
'
    Dim HT_bIndex As Byte System                        ' Pointer within the string to display
    Dim HT_bFontLine As Byte System                     ' Position within the font for the line of pixels

    Dim HT_bScroll_StartX As Byte                       ' Holds the X position For the start of the scrolling area
    Dim HT_bScroll_EndX As Byte                         ' Holds the X position For the End of the scrolling area
    Dim HT_bScroll_StartY As Byte                       ' Holds the Y position of the Line To scroll

    Dim bShadowRam[96] As Byte                          ' Holds the shadow display RAM

    Symbol cXpos_Max = 23                               ' Maximum X position of display (0 to 23)
    Symbol cYpos_Max = 15                               ' Maximum Y position of display (0 to 15)

    Symbol Switch = PORTB.7                             ' Where the button is attached(active low)

'-------------------------------------------------------------------------------------
    GoTo Main                                           ' Jump over the subroutines
'-------------------------------------------------------------------------------------
' Set or clear a shadow display pixel
' Input     : HT_bSetPixelX holds the X position of the pixel
'             HT_bSetPixelY holds the Y position of the Pixel
' Output    : None
' Notes     : Plot a point in the shadow display RAM
'             With the upper left hand corner being (0,0)
'             And the lower right hand corner being (23, 15).
'
$define Shadow_Plot(pX, pY) '
    HT_bSetPixelX = pX      '
    WREG = pY               '
    GoSub _Shadow_Plot

_Shadow_Plot:
    HT_bSetPixelY = WREG                                ' Transfer WREG into it's working variable
    If HT_bSetPixelY.7 = 1 Then Return                  ' \
    If HT_bSetPixelY > cHT_YMax Then Return             ' |
    If HT_bSetPixelX.7 = 1 Then Return                  ' | Boundary checks
    If HT_bSetPixelX > cHT_XMax Then Return             ' /
'
' The 4 bits in a single memory word go down, with the LSB (last transmitted) bit being on top.
' However, HT_ShiftOut() sends the LSB first, so we have to do a bit-reversal somewhere.
' This is done by shifting the single bit in the opposite direction from what you might expect.
'
    HT_bBitVal = HT_bSetPixelY & 3                      ' \ Calculate which bit to act upon
    HT_bBitVal = 8 >> HT_bBitVal                        ' /
    HT_bAddress = HT_bSetPixelY / 4                     ' \
    PRODL = HT_bSetPixelX * 4                           ' | Calculate which memory word this is in
    HT_bAddress = HT_bAddress + PRODL                   ' /
    PRODL = bShadowRam[HT_bAddress]                     ' fetch from shadow display RAM
    If tShadowPixelColor = tPixelSet Then               ' Do we need to set a shadow pixel?
        PRODH = PRODL | HT_bBitVal                      ' Yes. So set the shadow pixel
    Else                                                ' Otherwise...
        WREG = ~HT_bBitVal                              ' \
        PRODH = PRODL & WREG                            ' / Clear the shadow pixel
    EndIf
    bShadowRam[HT_bAddress] = PRODH                     ' Place the new value into the shadow RAM
    Return

'-------------------------------------------------------------------------------------
' Read a shadow display pixel's state
' Input     : HT_bSetPixelX holds the X position of the pixel
'             HT_bSetPixelY holds the Y position of the pixel
' Output    : WREG holds 1 or 0 depending on whether the shadow display's pixel is set or clear
'             Returns with WREG holding 0 if out of bounds
' Notes     : None
'
$define Shadow_Pixel(pX, pY, pReturn) '
    HT_bSetPixelX = pX                '
    WREG = pY                         '
    GoSub _Shadow_Pixel               '
    pReturn = WREG

_Shadow_Pixel:
    HT_bSetPixelY = WREG                                ' Transfer WREG into it's working variable
    WREG = 0                                            ' Clear return ready for a boundary overflow
    If HT_bSetPixelY.7 = 1 Then Return                  ' \
    If HT_bSetPixelY > cHT_YMax Then Return             ' | Boundary checks
    If HT_bSetPixelX.7 = 1 Then Return                  ' | Return holding 0 if out of bounds
    If HT_bSetPixelX > cHT_XMax Then Return             ' /

    HT_bBitVal = HT_bSetPixelY & 3                      ' \ Calculate which bit to act upon
    HT_bBitVal = 8 >> HT_bBitVal                        ' /
    HT_bAddress = HT_bSetPixelY / 4                     ' \
    PRODL = HT_bSetPixelX * 4                           ' | Calculate which memory word this is in
    HT_bAddress = HT_bAddress + PRODL                   ' /
    PRODL = bShadowRam[HT_bAddress]                     ' Fetch from shadow display RAM
    WREG = PRODL & HT_bBitVal
    If WREG > 0 Then WREG = 1
    Return

'-------------------------------------------------------------------------------------
' Scroll Left, n lines on the display
' Input     : HT_bScroll_StartY holds the Y position of the line to scroll
'             HT_bScroll_StartX holds the X position for the start of the scrolling area
'             HT_bScroll_EndX holds the X position for the end of the scrolling area
' Output    : None
' Notes     : None
'
Scroll_Left:
    HT_bCol = HT_bScroll_EndX
    Repeat
        HT_bRow = 7
        Repeat
            HT_Pixel(HT_bCol + 1,HT_bScroll_StartY + HT_bRow, tPixelColor) ' Read a pixel
            HT_Plot(HT_bCol,HT_bScroll_StartY + HT_bRow)   ' Place the pixel one position left
            Dec HT_bRow
        Until HT_bRow = 0
        Inc HT_bCol
    Until HT_bCol > HT_bScroll_StartX
    Return

'-------------------------------------------------------------------------------------
' Scroll a message across the display
' Input     : Cdata table "Message_Text", holds the text to display (null terminated)
' Output    : None
' Notes     : Re-uses a lot of variables declared in the "2416.inc" file
'
ScrollMessage:
    HT_tPixelColSave = tPixelColor                  ' Save the tPixelColor value
    TABLAT = CRead8 Message_Text[HT_bIndex]         ' Get a character from the string
    '
    ' Display a line of pixels from the character
    '
    HT_wCharOffset = TABLAT * 6                     ' 6 horizontal bytes per font
    HT_wCharOffset = HT_wCharOffset + HT_bFontLine  ' Add the offset to the character's value
    TABLAT = CRead8 CharFont[HT_wCharOffset]      ' Read the byte from the font table
'
' Send the value in TABLAT to the Display as a line of vertical pixels
'
    HT_bRow = 7                                     ' 7 vertical pixels per font byte
    Repeat
        tPixelColor = TABLAT.6
        HT_Plot(HT_bScroll_StartX,HT_bScroll_StartY + HT_bRow)
        TABLAT = TABLAT << 1
        Dec HT_bRow
    Until HT_bRow = 0
    Inc HT_bFontLine                                ' Move to the next line of pixels in the character
    GoSub Scroll_Left                               ' Scroll the line of text left by one pixel
    If HT_bFontLine >= 6 Then                       ' Have we done 6 pixels?
        HT_bFontLine = 0                            ' Yes. So reset the counter
        Inc HT_bIndex                               ' Move to the next character in the string
        If HT_bIndex >= Len(Message_Text) Then      ' Have we reached the end of the string?
            HT_bIndex = 0                           ' Yes. So start at the beginning
        EndIf
    EndIf
    tPixelColor = HT_tPixelColSave                  ' Restore the tPixelColor value
    Return

'----------------------------------------------------------------------------------
' Check for a stationary display
' Input     : None
' Output    : bDifferenceCount holds 0 if the display is static
' Notes     : None
'
$define CheckForStaticDisplay() GoSub _CheckForStaticDisplay

_CheckForStaticDisplay:
    bDifferenceCount = 0
    For Ypos = 0 To cYpos_Max                               ' Create a loop for the size of the Y axis
        For Xpos = 0 To cXpos_Max                           ' Create a loop for the size of the X axis
            HT_Pixel(Xpos, Ypos, tPixelColor)               ' Read from the display into tPixelColor
            Shadow_Pixel(Xpos, Ypos, bNeighbourState)       ' Read the shadow display into bNeighbourState
            If tPixelColor <> bNeighbourState Then          ' Are they both the same
                Inc bDifferenceCount                        ' No. So increment a difference counter
            EndIf
        Next                                                ' Close the x loop
    Next                                                    ' Close the Y loop
    Return

'----------------------------------------------------------------------------------
' Generate a pseudo random pattern of pixels on the display
' Input     : None
' Output    : bShadowRam holds a copy of the actual display
' Notes     : None
'
$define Generate() GoSub _Generate

_Generate:
    For Ypos = 1 To cYpos_Max - 1                               ' Scan the Y position
        For Xpos = 1 To cXpos_Max - 1                           ' Scan the X position
            wRandResult = Random                                ' Generate a pseudo random value between 1 and 65535
            Select wRandResult
                Case 48000 To 56000                             ' Look for a particular value
                    tShadowPixelColor = 1                       ' \
                    Shadow_Plot(Xpos,Ypos)                      ' / Set the shadow RAM's pixel
                    HT_Plot(Xpos,Ypos)                          ' Set the actual display's pixel
            EndSelect
        Next
    Next
    Return

'----------------------------------------------------------------------------------
' Transfer the shadow display RAM to the actual display
' Input     : bShadowRam
' Output    : None
' Notes     : None
'
$define Transfer_Generation() GoSub _Transfer_Generation

_Transfer_Generation:
    For Ypos = 0 To cYpos_Max
        For Xpos = 0 To cXpos_Max
            Shadow_Pixel(Xpos, Ypos, tPixelColor)       ' Read from the shadow display into tPixelColor
            HT_Plot(Xpos,Ypos)                          ' Plot the resulting pixel colour
        Next
    Next
    Return

'----------------------------------------------------------------------------------
' Check a neighbour cell (pixel) for live status
' Input     : HT_bSetPixelX holds the X position of the pixel
'             HT_bSetPixelY holds the Y position of the Pixel
' Output    : bNeighbours is incremented if the cell is alive
' Notes     : HT_bSetPixelX and HT_bSetPixelY are declared in the "2416.inc" file
'
$define CheckNeighbour(pXpos, pYpos) '
    HT_bSetPixelX = pXpos            '
    HT_bSetPixelY = pYpos            '
    GoSub _CheckNeighbour

_CheckNeighbour:
    HT_Pixel(HT_bSetPixelX, HT_bSetPixelY, bNeighbourState) ' Read the cell
    If bNeighbourState = 1 Then Inc bNeighbours             ' Increment the Neighbours counter if alive
    Return

'----------------------------------------------------------------------------------
' Calculate a single generation of cells
' Input     : None
' Output    : bShadowRam holds a copy of the altered colonies
' Notes     : Any live cell with fewer than two live neighbours dies, as if caused by underpopulation.
'             Any live cell with more than three live neighbours dies, as if by overcrowding.
'             Any live cell with two or three live neighbours lives on to the next generation.
'             Any dead cell with exactly three live neighbours becomes a live cell.
'
$define Calculate_Generations() GoSub _Calculate_Generations

_Calculate_Generations:
    For Ypos = 0 To cYpos_Max                               ' Scan the Y position
        For Xpos = 0 To cXpos_Max                           ' Scan the X position
            bNeighbours = 0                                 ' Clear the bNeighbours counter
            CheckNeighbour(Xpos - 1, Ypos - 1)              ' Read the cell above and to the left
            CheckNeighbour(Xpos - 1, Ypos)                  ' Read the cell left
            CheckNeighbour(Xpos - 1, Ypos + 1)              ' Read the cell above and to the right
            CheckNeighbour(Xpos, Ypos - 1)                  ' Read the cell to the left
            CheckNeighbour(Xpos, Ypos + 1)                  ' Read the cell below
            CheckNeighbour(Xpos + 1, Ypos - 1)              ' Read the cell below and to the left
            CheckNeighbour(Xpos + 1, Ypos)                  ' Read the cell below
            CheckNeighbour(Xpos + 1, Ypos + 1)              ' Read the cell below and to the right

            HT_Pixel(Xpos, Ypos, bNeighbourState)           ' Read the current location of the cell into bNeighbourState

            If bNeighbourState = 1 Then                         ' Is the current cell alive?
                If bNeighbours <> 3 And bNeighbours <> 2 Then   ' Yes. So are there two or three live neighbours
                    tShadowPixelColor = 0                       ' \ Yes. So the cell has to die
                    Shadow_Plot(Xpos,Ypos)                      ' / Place the outcome in the shadow display RAM
                EndIf
            Else                                                ' Otherwise. the cell must be dead
                If bNeighbours = 3 Then                         ' Does the cell have 3 live neighbours?
                    tShadowPixelColor = 1                       ' \ Yes. So resurrect the cell
                    Shadow_Plot(Xpos,Ypos)                      ' / Place the outcome in the shadow display RAM
                EndIf
            EndIf
        Next
    Next
    Return

'----------------------------------------------------------------------------------
' Write scrolling text across the display until the button is pressed
'
$define WaitForButton() GoSub _WaitForButton

_WaitForButton:
    HT_Line(0,0,23,0)                                   ' \
    HT_Line(23,0,23,15)                                 ' | Draw a line around the display
    HT_Line(23,15,0,15)                                 ' |
    HT_Line(0,15,0,0)                                   ' /
    While Switch = 1                                    ' Loop until the button is pressed
        GoSub ScrollMessage                             ' Scroll the message
        DelayMS 50                                      ' Delay so we can see the text scrolling
    Wend
    Return

'----------------------------------------------------------------------------------
' The main program loop starts here
'
Main:
    HT_bIndex = 0                                       ' Reset the scrolling tex string pointer
    HT_bFontLine = 0                                    ' Reset the text font line counter
    HT_bScroll_StartY = 3                               ' Set scrolling text Y position to 3
    HT_bScroll_StartX = 21                              ' Start scrolling at pixel 21
    HT_bScroll_EndX = 2                                 ' End the scroll at position 2
    HT_Clear()                                          ' Clear the display
    Clear bShadowRam                                    ' Clear the shadow display RAM
    Input Switch                                        ' Make the button pin an input
'
' Initialise Timer1 for:
'                        No Interrupt on Timer1 overflow
'                        16-bit read/write mode
'                        Internal clock source
'                        1:8 Prescaler
'
    OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_8)
'
' Wait for the button to be pressed
'
    WaitForButton()                                     ' Wait for a button press
    wRandResult = ReadTimer1()                          ' Read the value of Timer1
    Seed wRandResult                                    ' Seed the pseudo random generator with it
    Do                                                  ' Create an infinite loop
        Generate()                                      ' Generate a colony of cells
        While                                           ' Create a loop for the generations
            Transfer_Generation()                       ' Transfer the shadow display to the actual display
            Calculate_Generations()                     ' Calculate a single generation
            CheckForStaticDisplay()                     ' Check for a static display
            If bDifferenceCount = 0 Then                ' Is the display static?
                DelayMS 500                             ' Yes. So delay for a while
                Break                                   ' Exit the loop
            EndIf
            If Switch = 0 Then                          ' Has the button been pressed?
                DelayMS 300                             ' Yes. So delay a while
                Break                                   ' Exit the loop
            EndIf
            DelayMS 170                                 ' Delay so we can see what's happening
        Wend                                            ' Infinite generations
        DelayMS 300                                     ' A small delay
    Loop                                                ' Do it forever
'-------------------------------------------------------------------------------------
'
' The text to scroll on the display
' Must be null terminated
' Must be less than or equal to 255 characters in length
'
Message_Text:
    CData "** The Game of Life. Press the Button to Start ** ", 0
'
' Character Font in a flash memory table
'
CharFont:
    CData    $00,$00,$00,$00,$00,$00,_      'Graphic character 0
             $FF,$FF,$FF,$FF,$FF,$FF,_      'Graphic character 1
             $07,$07,$07,$00,$00,$00,_      'Graphic character 2
             $00,$00,$00,$07,$07,$07,_      'Graphic character 3
             $E0,$E0,$E0,$00,$00,$00,_      'Graphic character 4
             $00,$00,$00,$E0,$E0,$E0,_      'Graphic character 5
             $FF,$FF,$FF,$00,$00,$00,_      'Graphic character 6
             $00,$00,$00,$FF,$FF,$FF,_      'Graphic character 7
             $07,$07,$07,$07,$07,$07,_      'Graphic character 8
             $E0,$E0,$E0,$E0,$E0,$E0,_      'Graphic character 9
             $E0,$E0,$E0,$07,$07,$07,_      'Graphic character 10
             $07,$07,$07,$E0,$E0,$E0,_      'Graphic character 11
             $FF,$FF,$FF,$E0,$E0,$E0,_      'Graphic character 12
             $E0,$E0,$E0,$FF,$FF,$FF,_      'Graphic character 13
             $07,$07,$07,$FF,$FF,$FF,_      'Graphic character 14
             $FF,$FF,$FF,$07,$07,$07,_      'Graphic character 15
             $55,$AA,$55,$AA,$55,$AA,_      'Graphic character 16
             $AA,$55,$AA,$55,$AA,$55,_      'Graphic character 17
             $01,$01,$01,$01,$01,$01,_      'Graphic character 18
             $80,$80,$80,$80,$80,$80,_      'Graphic character 19
             $FF,$00,$00,$00,$00,$00,_      'Graphic character 20
             $00,$00,$00,$00,$00,$FF,_      'Graphic character 21
             $FF,$01,$01,$01,$01,$01,_      'Graphic character 22
             $01,$01,$01,$01,$01,$FF,_      'Graphic character 23
             $FF,$80,$80,$80,$80,$80,_      'Graphic character 24
             $80,$80,$80,$80,$80,$FF,_      'Graphic character 25
             $00,$00,$00,$00,$F0,$F0,_      'User defined character 26
             $00,$00,$00,$00,$0F,$0F,_      'User defined character 27
             $00,$00,$00,$00,$00,$00,_      'User defined character 28
             $00,$00,$00,$00,$00,$00,_      'User defined character 29
             $00,$00,$00,$00,$00,$00,_      'User defined character 30
             $00,$00,$00,$00,$00,$00,_      'User defined character 31
             $00,$00,$00,$00,$00,$00,_      '32 -   - 20        
             $00,$00,$4F,$00,$00,$00,_      '33 - ! - 21
             $00,$07,$00,$07,$00,$00,_      '34 - " - 22
             $14,$7F,$14,$7F,$14,$00,_      '35 - # - 23
             $24,$2A,$7F,$2A,$12,$00,_      '36 - $ - 24
             $23,$13,$08,$64,$62,$00,_      '37 - % - 25
             $36,$49,$55,$22,$50,$00,_      '38 - & - 26
             $00,$05,$03,$00,$00,$00,_      '39 - ' - 27
             $1C,$22,$41,$00,$00,$00,_      '40 - ( - 28
             $00,$00,$41,$22,$1C,$00,_      '41 - ) - 29
             $14,$08,$3E,$08,$14,$00,_      '42 - * - 2A
             $08,$08,$3E,$08,$08,$00,_      '43 - + - 2B
             $00,$50,$30,$00,$00,$00,_      '44 - , - 2C
             $08,$08,$08,$08,$08,$00,_      '45 - - - 2D
             $00,$60,$60,$00,$00,$00,_      '46 - . - 2E
             $20,$10,$08,$04,$02,$00,_      '47 - / - 2F
             $3E,$51,$49,$45,$3E,$00,_      '48 - 0 - 30
             $00,$42,$7F,$40,$00,$00,_      '49 - 1 - 31
             $42,$61,$51,$49,$46,$00,_      '50 - 2 - 32
             $21,$41,$45,$4B,$31,$00,_      '51 - 3 - 33
             $18,$14,$12,$7F,$10,$00,_      '52 - 4 - 34
             $27,$45,$45,$45,$39,$00,_      '53 - 5 - 35
             $3C,$4A,$49,$49,$30,$00,_      '54 - 6 - 36
             $01,$71,$09,$05,$03,$00,_      '55 - 7 - 37
             $36,$49,$49,$49,$36,$00,_      '56 - 8 - 38
             $06,$49,$49,$49,$3E,$00,_      '57 - 9 - 39
             $00,$36,$36,$00,$00,$00,_      '58 - : - 3A
             $00,$56,$36,$00,$00,$00,_      '59 - ; - 3B
             $08,$14,$22,$41,$00,$00,_      '60 - < - 3C
             $14,$14,$14,$14,$14,$00,_      '61 - = - 3D
             $00,$41,$22,$14,$08,$00,_      '62 - > - 3E
             $02,$01,$51,$09,$06,$00,_      '63 - ? - 3F
             $32,$49,$79,$41,$3E,$00,_      '64 - @ - 40
             $7E,$11,$11,$11,$7E,$00,_      '65 - A - 41
             $7F,$49,$49,$49,$36,$00,_      '66 - B - 42
             $3E,$41,$41,$41,$22,$00,_      '67 - C - 43
             $7F,$41,$41,$22,$1C,$00,_      '68 - D - 44
             $7F,$49,$49,$49,$41,$00,_      '69 - E - 45
             $7F,$09,$09,$09,$01,$00,_      '70 - F - 46
             $3E,$41,$49,$49,$7A,$00,_      '71 - G - 47
             $7F,$08,$08,$08,$7F,$00,_      '72 - H - 48
             $00,$41,$7F,$41,$00,$00,_      '73 - I - 49
             $20,$40,$41,$3F,$01,$00,_      '74 - J - 4A
             $7F,$08,$14,$22,$41,$00,_      '75 - K - 4B
             $7F,$40,$40,$40,$40,$00,_      '76 - L - 4C
             $7F,$02,$0C,$02,$7F,$00,_      '77 - M - 4D
             $7F,$04,$08,$10,$7F,$00,_      '78 - N - 4E
             $3E,$41,$41,$41,$3E,$00,_      '79 - O - 4F
             $7F,$09,$09,$09,$06,$00,_      '80 - P - 50
             $3E,$41,$51,$21,$5E,$00,_      '81 - Q - 51
             $7F,$09,$19,$29,$46,$00,_      '82 - R - 52
             $46,$49,$49,$49,$31,$00,_      '83 - S - 53
             $01,$01,$7F,$01,$01,$00,_      '84 - T - 54
             $3F,$40,$40,$40,$3F,$00,_      '85 - U - 55
             $1F,$20,$40,$20,$1F,$00,_      '86 - V - 56
             $3F,$40,$38,$40,$3F,$00,_      '87 - W - 57
             $63,$14,$08,$14,$63,$00,_      '88 - X - 58
             $07,$08,$70,$08,$07,$00,_      '89 - Y - 59
             $61,$51,$49,$45,$43,$00,_      '90 - Z - 5A
             $7F,$41,$41,$00,$00,$00,_      '91 - [ - 5B
             $02,$04,$08,$10,$20,$00,_      '92 - \ - 5C
             $00,$00,$41,$41,$7F,$00,_      '93 - ] - 5D
             $04,$02,$01,$02,$04,$00,_      '94 - ^ - 5E
             $40,$40,$40,$40,$40,$00,_      '95 - _ - 5F
             $00,$01,$02,$04,$00,$00,_      '96 - ` - 60
             $20,$54,$54,$54,$78,$00,_      '97 - a - 61
             $7F,$48,$44,$44,$38,$00,_      '98 - b - 62
             $38,$44,$44,$44,$20,$00,_      '99 - c - 63
             $38,$44,$44,$48,$7F,$00,_      '100  d - 64
             $38,$54,$54,$54,$18,$00,_      '101  e - 65
             $08,$7E,$09,$01,$02,$00,_      '102  f - 66
             $0C,$52,$52,$52,$3E,$00,_      '103  g - 67
             $7F,$08,$04,$04,$78,$00,_      '104  h - 68
             $00,$44,$7D,$40,$00,$00,_      '105  i - 69
             $00,$20,$40,$44,$3D,$00,_      '106  j - 6A
             $7F,$10,$28,$44,$00,$00,_      '107  k - 6B
             $00,$41,$7F,$40,$00,$00,_      '108  l - 6C
             $7C,$04,$18,$04,$78,$00,_      '109  m - 6D
             $7C,$08,$04,$04,$78,$00,_      '110  n - 6E
             $38,$44,$44,$44,$38,$00,_      '111  o - 6F
             $7C,$14,$14,$14,$08,$00,_      '112  p - 70
             $08,$14,$14,$18,$7C,$00,_      '113  q - 71
             $7C,$08,$04,$04,$08,$00,_      '114  r - 72
             $48,$54,$54,$54,$20,$00,_      '115  s - 73
             $04,$3F,$44,$40,$20,$00,_      '116  t - 74
             $3C,$40,$40,$20,$7C,$00,_      '117  u - 75
             $1C,$20,$40,$20,$1C,$00,_      '118  v - 76
             $3C,$40,$30,$40,$3C,$00,_      '119  w - 77
             $44,$28,$10,$28,$44,$00,_      '120  x - 78
             $0C,$50,$50,$50,$3C,$00,_      '121  y - 79
             $44,$64,$54,$4C,$44,$00,_      '122  z - 7A
             $08,$36,$41,$00,$00,$00,_      '123  { - 7B
             $00,$00,$7F,$00,$00,$00,_      '124  | - 7C
             $00,$00,$41,$36,$08,$00,_      '125  } - 7D
             $00,$08,$04,$08,$04,$00        '126  ~ - 7E


